close all;
clear all;
clf;
clc;

%This is the time signal vector we are going to transform. You should
%change this to whatever you want, note that it will dynamically rescale to
%any arbitrary length :)
%vector = [0, 1, 2, 3, 4, 5, 6, 7] % a ramp vector
%vector = sin(2*pi*[1:1:25]/5) %a sine function with a perfect periodic extension
vector = sin(2*pi*[1:1:100]/30) %a sine function where the periodic extension is in error
%vector = round(2*randn(1,6)); %this function generates a random vector

%This is the length of our vector to transform.
size = length(vector);
%We create the transformation matrix of the right size for our vector.
dft_mtx = dftmtx(size);
%Linearly create the omega values for each element.
%EG: For size = 4 the vector is:    2*pi*[0, 1/4, 2/4, 3/4]
omega = linspace(0, 1 - (1 / size), size) * 2 * pi;
midpoint = ceil(size/2)+2;

%This is just time. Something for us to plot against. I made the size equal
%to two periods, so we can see the repeating nature of our output.
t = linspace(0, 2 * size, 1000);
%Here we compute the transform. Note that it gives real and imaginary
%numbers. If we were to graph the frequency domain like in Fourier
%Transforms, we would graph the absolute values of these.
transform = (vector * dft_mtx) / size



%move the basis vectors to be sitting over the interval -pi to pi. the DFT
%and FFT are defined across the interval 0->2pi, as it makes the math
%easier (remember that the section from pi->2pi is identical to -pi-> 0 ), 
%but we need to ensure our reconstruction happens over the lower interval 
%to keep the reconstruction confined to the Bandwith 0->pi.
for i = midpoint:1:size
    omega(i) = omega(i)-2*pi;
end
%Now we interpolate the signal in the time domain, via sinc reconstruction
%to visualise the process. In order to do this, we recover the continuous(ish)
%response by taking each succesive coefficient of our transform, spliting 
%it into real and imaginary components, and computing:
%v(t) real*cos(wt) - imag*i*sin(wt).
visualisation = zeros(1, 1000);

%the first 4 terms of the reconstruction
figure('Name','The first four reconstructions','NumberTitle','off')
for x = 1:4
    visualisation = visualisation + real(transform(x))*cos(omega(x)*t) - imag(transform(x))*sin(omega(x)*t) ;
    subplot(2,2,x)
    hold on
    plot(t, real( visualisation), 'r', 'Linewidth', 2);
    stem(t(1:1000/(2*size):end),[vector,vector],'k');
end


figure('Name','The final four reconstructions','NumberTitle','off')
%look at the final four terms of the reconstruction
visualisation = zeros(1, 1000);
for x = 1:size-4
    visualisation = visualisation + real(transform(x))*cos(omega(x)*t) - imag(transform(x))*sin(omega(x)*t) ;
end
for x = size-3:size
    visualisation = visualisation + real(transform(x))*cos(omega(x)*t) - imag(transform(x))*sin(omega(x)*t) ;
    subplot(2,2,x-size+4)
    hold on
    plot(t, real( visualisation), 'r', 'Linewidth', 2);
    stem(t(1:1000/(2*size):end),[vector,vector],'k');
end


figure('Name','All intermediate reconstructions','NumberTitle','off')
hold on
stem(t(1:1000/(2*size):end),[vector,vector],'k');
tdisc = t(1:1000/(2*size):end);
visualisation = zeros(1, 1000);
currDiscreteRecon = zeros(1,length(tdisc));
msqerr = [];
%all steps of reconstruction overlaid
for x = 1:size
    visualisation = visualisation + real(transform(x))*cos(omega(x)*t) - imag(transform(x))*sin(omega(x)*t) ;
    plot(t, real( visualisation), 'r', 'Linewidth', 2);
    currDiscreteRecon = currDiscreteRecon + real(transform(x))*cos(omega(x)*tdisc) - imag(transform(x))*sin(omega(x)*tdisc) ;
    msqerr(x) = norm(currDiscreteRecon-[vector,vector])/2;
end

figure('Name','Reconstruction of an Arbitrary Vector using only Sinusoids','NumberTitle','off')
%The reconstruction on its own. Note that the interpolation using every
%basis function perfectly intercects each sample. This is garunteed for the
%fourier basis, and indeed any linearly independent basis. 
hold on
plot(t, real(visualisation), 'r', 'Linewidth', 2);
stem(t(1:1000/(2*size):end),[vector,vector],'k');
xlabel('Time (s)');
ylabel('Arbitrary Units');
title('Reconstruction of an Arbitrary Vector using only Sinusoids');
legend('Fourier reconstruction','Discrete Samples', 'Location', 'best');
hold off


%Additionally, because the Fourier basis is orthogonal, the mean squared 
%error is garunteed to be lower with each successive basis function
%included in the reconstruction. this is only garunteed with an orthogonal
%basis. If this wasn't the case filtering would not be a simple point wise
%mulitplication in Frequency, as you would need to recompute the basis
%weights when you want to reduce/eliminate a particular frequency.
figure('Name','Mean Sqared error of successive reconstructions','NumberTitle','off')
stem(1:size,msqerr,'k');
xlabel('number of basis functions');
ylabel('Mean Squared error ');