function [DFT, IFFT] = computeDFTBasis( N )
%%this script computes the DFT and IDFT basis for an N point vector.
%its intended to serve as an example - note that the functions FFT and 
%IFFT give the exact same result but significantly more efficently

t = 1:1:N;
t = t-1;
w = t/N;

%note the order of calculations is N*N, and the DFT is an algorithm of
%O(n^2). the FFT is O(n*log2(n)), scaling significantly better. by ~1000
%points its on the order of 100x better than the direct DFT, by 1,000,000
%it is ~50,000x faster
for time = 1:1:N
    for omega = 1:1:N
        %the DFT basis function equation - DFT*v where v is a column vector matches the output of matlabs FFT. note that the function is unscaled
        DFT(omega,time) = exp(-2*pi()*i*w(omega)*t(time));
   
        %the inverse of the DFT, again this returns the exact same result as the IFFT.
        %note the 1/N scaling, and that it is identical to the DFT aside from the
        %argument of the exponential function changing signs
        IFFT(omega,time) = exp(2*pi()*i*w(omega)*t(time))*1/N; 
    end
end

%or you could use matlabs dftmtx function. (matlab <3)
%dftmtx(N)
end
